==> hardware-monitor-0.1.tar.gz: hardware-monitor-0.1/src/monitor-impls.cpp <==
/* The various system statistics - adapters of the libgtop interface.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <config.h>

#include <string>
#include <sstream>
#include <iomanip>
#include <ostream>
#include <sys/time.h>	      // for high-precision timing for the network load

#include <glibtop.h>
#include <glibtop/cpu.h>
#include <glibtop/mem.h>
#include <glibtop/swap.h>
#include <glibtop/loadavg.h>
#include <glibtop/fsusage.h>
#include <glibtop/netload.h>

#include "monitor-impls.hpp"
#include "ucompose.hpp"
#include "i18n.hpp"


// FIXME: put this in configure? investigate diskusage applet

#define DISK_BLOCK_SIZE 1024
#define disk_block_size DISK_BLOCK_SIZE

// helpers

// for setting precision
struct Precision
{
  int n;
};

std::ostream &operator <<(std::ostream& os, const Precision &p)
{
  os << std::setprecision(p.n) << std::setiosflags(std::ios_base::fixed);
  return os;
}

Precision precision(int n)
{
  Precision p;
  p.n = n;
  return p;
}

// for getting max no. of decimal digits
Precision decimal_digits(double val, int n)
{
  Precision p;

  if (val == 0)
    p.n = 1;
  else {
    while (val > 1 && n > 0) {
      val /= 10;
      --n;
    }

    p.n = n;
  }
  
  return p;
}


//
// class CpuUsageMonitor
//

CpuUsageMonitor::CpuUsageMonitor()
  : cpu_no(all_cpus), user_time(0), nice_time(0), sys_time(0), idle_time(0)
{}

CpuUsageMonitor::CpuUsageMonitor(int cpu)
  : cpu_no(cpu), user_time(0), nice_time(0), sys_time(0), idle_time(0)
{
  assert(cpu_no >= 0 && cpu_no < GLIBTOP_NCPU);
}

double CpuUsageMonitor::value()
{
  glibtop_cpu cpu;
	
  glibtop_get_cpu(&cpu);

  guint64 u, n, s, i;
	
  if (cpu_no == all_cpus) {
    u = cpu.user;
    n = cpu.nice;
    s = cpu.sys;
    i = cpu.idle;
  }
  else {
    u = cpu.xcpu_user[cpu_no];
    n = cpu.xcpu_nice[cpu_no];
    s = cpu.xcpu_sys[cpu_no];
    i = cpu.xcpu_idle[cpu_no];
  }
	
  // calculate ticks since last call
  guint64
    duser = u - user_time,
    dnice = n - nice_time,
    dsys = s - sys_time,
    didle = i - idle_time; 

  // and save the new values
  user_time = u;
  nice_time = n;
  sys_time = s;
  idle_time = i;
    
  guint64 total = duser + dnice + dsys + didle;

  if (total > 0)
    // don't count dnice since to avoid always showing 100% with SETI@home and
    // similar applications running
    return double(duser + dsys) / total;
  else
    return 0;
}

double CpuUsageMonitor::max()
{
  return 1;
}

Glib::ustring CpuUsageMonitor::format_value(double val)
{
  return String::ucompose(_("%1%%"), precision(1), 100 * val);
}

Glib::ustring CpuUsageMonitor::get_name()
{
  if (cpu_no == all_cpus)
    return _("All processors");
  else
    return String::ucompose(_("Processor no. %1"), cpu_no + 1);
}

Glib::ustring CpuUsageMonitor::get_short_name()
{
  if (cpu_no == all_cpus)
    // must be short
    return _("CPU");
  else
    // note to translators: %1 is the cpu no, e.g. "CPU 1"
    return String::ucompose(_("CPU %1"), cpu_no + 1);
}

void CpuUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			   const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("cpu_usage"));
  client->set(dir + "/cpu_no", cpu_no);
}


//
// class SwapUsageMonitor
//

SwapUsageMonitor::SwapUsageMonitor()
  : max_value(0)
{
}

double SwapUsageMonitor::value()
{
  glibtop_swap swp;
	
  glibtop_get_swap(&swp);
	
  max_value = swp.total;

  if (swp.total > 0)
    return swp.used;
  else
    return 0;
}

double SwapUsageMonitor::max()
{
  return max_value;
}

Glib::ustring SwapUsageMonitor::format_value(double val)
{
  val /= 1000000;
  
  return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
}

Glib::ustring SwapUsageMonitor::get_name()
{
  return _("Disk-based memory");
}

Glib::ustring SwapUsageMonitor::get_short_name()
{
  // must be short
  return _("Swap");
}

void SwapUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			    const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("swap_usage"));
}


//
// class LoadAverageMonitor
//

LoadAverageMonitor::LoadAverageMonitor()
  : max_value(0)
{
}

double LoadAverageMonitor::value()
{
  glibtop_loadavg loadavg;
	
  glibtop_get_loadavg (&loadavg);
	
  if (max_value < loadavg.loadavg[0])
    max_value = loadavg.loadavg[0];

  if (max_value > 0)
    return loadavg.loadavg[0];
  else
    return 0;
}

double LoadAverageMonitor::max()
{
  return max_value;
}

Glib::ustring LoadAverageMonitor::format_value(double val)
{
  return String::ucompose("%1", precision(1), val);
}

Glib::ustring LoadAverageMonitor::get_name()
{
  return _("Load average");
}

Glib::ustring LoadAverageMonitor::get_short_name()
{
  // note to translators: short for "load average" - it has nothing to do with
  // loading data
  return _("Load");
}

void LoadAverageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("load_average"));
}


//
// class MemoryUsageMonitor
//

MemoryUsageMonitor::MemoryUsageMonitor()
  : max_value(0)
{
}

double MemoryUsageMonitor::value()
{
  glibtop_mem mem;
	
  glibtop_get_mem (&mem);

  max_value = mem.total;

  if (mem.total > 0)
    return mem.used - (mem.buffer + mem.cached);
  else
    return 0;
}

double MemoryUsageMonitor::max()
{
  return max_value;
}
	
Glib::ustring MemoryUsageMonitor::format_value(double val)
{
  val /= 1000000;
  
  return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
}

Glib::ustring MemoryUsageMonitor::get_name()
{
  return _("Memory");
}

Glib::ustring MemoryUsageMonitor::get_short_name()
{
  // short for memory
  return _("Mem.");
}

void MemoryUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("memory_usage"));
}


//
// class DiskUsageMonitor
//

DiskUsageMonitor::DiskUsageMonitor(const std::string &dir)
  : max_value(0), mount_dir(dir)
{
}

double DiskUsageMonitor::value()
{
  glibtop_fsusage fsusage;

  glibtop_get_fsusage(&fsusage, mount_dir.c_str());

  max_value = fsusage.blocks;

  if (fsusage.blocks > 0)
    return fsusage.bavail;
  else
    return 0;
}

double DiskUsageMonitor::max()
{
  return max_value;
}

Glib::ustring DiskUsageMonitor::format_value(double val)
{
  // 1 block = disk_block_size bytes
  val *= disk_block_size;

  if (val >= 1000 * 1000 * 1000) {
    val /= 1000 * 1000 * 1000;
    return String::ucompose(_("%1 Gb"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000 * 1000) {
    val /= 1000 * 1000;
    return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000) {
    val /= 1000;
    return String::ucompose(_("%1 kb"), decimal_digits(val, 3), val);
  }
  else
    return String::ucompose(_("%1 b"), decimal_digits(val, 3), val);
}

Glib::ustring DiskUsageMonitor::get_name()
{
  return String::ucompose(_("Disk (%1)"), mount_dir);
}


Glib::ustring DiskUsageMonitor::get_short_name()
{
  return String::ucompose("%1", mount_dir);
}

void DiskUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			    const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("disk_usage"));
  client->set(dir + "/mount_dir", mount_dir);
}


//
// class NetworkLoadMonitor
//

NetworkLoadMonitor::NetworkLoadMonitor(const Glib::ustring &inter, int inter_no,
				       Direction dir)
  : max_value(1), byte_count(0), time_stamp_secs(0), time_stamp_usecs(0),
    interface(inter), interface_no(inter_no), direction(dir)
{
}

double NetworkLoadMonitor::value()
{
  glibtop_netload netload;

  glibtop_get_netload(&netload,
		      String::ucompose("%1%2", interface, interface_no).c_str());
  guint64 val, bytes;
    
  if (direction == all_data)
    bytes = netload.bytes_total;
  else if (direction == incoming_data)
    bytes = netload.bytes_in;
  else
    bytes = netload.bytes_in;
  
  if (byte_count == 0)
    val = 0;   // avoid calculating an enormous throughput initially
  else
    val = bytes - byte_count;

  byte_count = bytes;

  if (val > max_value)
    max_value = val;

  // calculate difference in msecs
  struct timeval tv;
  if (gettimeofday(&tv, 0) == 0) {
    time_difference =
      (tv.tv_sec - time_stamp_secs) * 1000 +
      (tv.tv_usec - time_stamp_usecs) / 1000;
    time_stamp_secs = tv.tv_sec;
    time_stamp_usecs = tv.tv_usec;
  }

  return val;
}

double NetworkLoadMonitor::max()
{
  return max_value;
}

Glib::ustring NetworkLoadMonitor::format_value(double val)
{
  // 1000 ms = 1 s
  val = val / time_difference * 1000;

  if (val >= 1000 * 1000 * 1000) {
    val /= 1000 * 1000 * 1000;
    return String::ucompose(_("%1 Gb/s"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000 * 1000) {
    val /= 1000 * 1000;
    return String::ucompose(_("%1 Mb/s"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000) {
    val /= 1000;
    return String::ucompose(_("%1 kb/s"), decimal_digits(val, 3), val);
  }
  else 
    return String::ucompose(_("%1 b/s"), decimal_digits(val, 3), val);
}

Glib::ustring NetworkLoadMonitor::get_name()
{
  if (interface == "eth" && interface_no == 0)
    return _("Ethernet (first)");
  else if (interface == "eth" && interface_no == 1)
    return _("Ethernet (second)");
  else if (interface == "eth" && interface_no == 2)
    return _("Ethernet (third)");
  else if (interface == "ppp" && interface_no == 0)
    return _("Modem");
  else if (interface == "slip" && interface_no == 0)
    return _("Serial link");
  else
    // unknown, someone must have been fiddling with GConf
    return String::ucompose("%1%2", interface, interface_no);
}

Glib::ustring NetworkLoadMonitor::get_short_name()
{
  if (interface == "eth")
    // short for an ethernet card
    return String::ucompose(_("Eth. %1"), interface_no + 1);
  else if (interface == "ppp" && interface_no == 0)
    // short for modem
    return _("Mod.");
  else if (interface == "slip" && interface_no == 0)
    // short for serial link
    return _("Ser.");
  else
    // unknown, someone must have been fiddling with GConf
    return String::ucompose("%1%2", interface, interface_no);
}

void NetworkLoadMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("network_load"));
  client->set(dir + "/interface", interface);
  client->set(dir + "/interface_no", interface_no);
  client->set(dir + "/interface_direction", int(direction));
}

==> hardware-monitor-0.2.tar.gz: hardware-monitor-0.2/src/monitor-impls.cpp <==
/* The various system statistics - adapters of the libgtop interface.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <config.h>

#include <string>
#include <sstream>
#include <iomanip>
#include <ostream>
#include <sys/time.h>	      // for high-precision timing for the network load

#include <glibtop.h>
#include <glibtop/cpu.h>
#include <glibtop/mem.h>
#include <glibtop/swap.h>
#include <glibtop/loadavg.h>
#include <glibtop/fsusage.h>
#include <glibtop/netload.h>

#include "monitor-impls.hpp"
#include "ucompose.hpp"
#include "i18n.hpp"


// FIXME: put this in configure? investigate diskusage applet

#define DISK_BLOCK_SIZE 1024
#define disk_block_size DISK_BLOCK_SIZE

// helpers

// for setting precision
struct Precision
{
  int n;
};

std::ostream &operator <<(std::ostream& os, const Precision &p)
{
  os << std::setprecision(p.n) << std::setiosflags(std::ios_base::fixed);
  return os;
}

Precision precision(int n)
{
  Precision p;
  p.n = n;
  return p;
}

// for getting max no. of decimal digits
Precision decimal_digits(double val, int n)
{
  Precision p;

  if (val == 0)
    p.n = 1;
  else {
    while (val > 1 && n > 0) {
      val /= 10;
      --n;
    }

    p.n = n;
  }
  
  return p;
}


//
// class CpuUsageMonitor
//

CpuUsageMonitor::CpuUsageMonitor()
  : cpu_no(all_cpus), user_time(0), nice_time(0), sys_time(0), idle_time(0)
{}

CpuUsageMonitor::CpuUsageMonitor(int cpu)
  : cpu_no(cpu), user_time(0), nice_time(0), sys_time(0), idle_time(0)
{
  assert(cpu_no >= 0 && cpu_no < GLIBTOP_NCPU);
}

double CpuUsageMonitor::value()
{
  glibtop_cpu cpu;
	
  glibtop_get_cpu(&cpu);

  guint64 u, n, s, i;
	
  if (cpu_no == all_cpus) {
    u = cpu.user;
    n = cpu.nice;
    s = cpu.sys;
    i = cpu.idle;
  }
  else {
    u = cpu.xcpu_user[cpu_no];
    n = cpu.xcpu_nice[cpu_no];
    s = cpu.xcpu_sys[cpu_no];
    i = cpu.xcpu_idle[cpu_no];
  }
	
  // calculate ticks since last call
  guint64
    duser = u - user_time,
    dnice = n - nice_time,
    dsys = s - sys_time,
    didle = i - idle_time; 

  // and save the new values
  user_time = u;
  nice_time = n;
  sys_time = s;
  idle_time = i;
    
  guint64 total = duser + dnice + dsys + didle;

  if (total > 0)
    // don't count dnice since to avoid always showing 100% with SETI@home and
    // similar applications running
    return double(duser + dsys) / total;
  else
    return 0;
}

double CpuUsageMonitor::max()
{
  return 1;
}

Glib::ustring CpuUsageMonitor::format_value(double val)
{
  return String::ucompose(_("%1%%"), precision(1), 100 * val);
}

Glib::ustring CpuUsageMonitor::get_name()
{
  if (cpu_no == all_cpus)
    return _("All processors");
  else
    return String::ucompose(_("Processor no. %1"), cpu_no + 1);
}

Glib::ustring CpuUsageMonitor::get_short_name()
{
  if (cpu_no == all_cpus)
    // must be short
    return _("CPU");
  else
    // note to translators: %1 is the cpu no, e.g. "CPU 1"
    return String::ucompose(_("CPU %1"), cpu_no + 1);
}

void CpuUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			   const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("cpu_usage"));
  client->set(dir + "/cpu_no", cpu_no);
}


//
// class SwapUsageMonitor
//

SwapUsageMonitor::SwapUsageMonitor()
  : max_value(0)
{
}

double SwapUsageMonitor::value()
{
  glibtop_swap swp;
	
  glibtop_get_swap(&swp);
	
  max_value = swp.total;

  if (swp.total > 0)
    return swp.used;
  else
    return 0;
}

double SwapUsageMonitor::max()
{
  return max_value;
}

Glib::ustring SwapUsageMonitor::format_value(double val)
{
  val /= 1000000;
  
  return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
}

Glib::ustring SwapUsageMonitor::get_name()
{
  return _("Disk-based memory");
}

Glib::ustring SwapUsageMonitor::get_short_name()
{
  // must be short
  return _("Swap");
}

void SwapUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			    const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("swap_usage"));
}


//
// class LoadAverageMonitor
//

LoadAverageMonitor::LoadAverageMonitor()
  : max_value(0)
{
}

double LoadAverageMonitor::value()
{
  glibtop_loadavg loadavg;
	
  glibtop_get_loadavg (&loadavg);
	
  if (max_value < loadavg.loadavg[0])
    max_value = loadavg.loadavg[0];

  if (max_value > 0)
    return loadavg.loadavg[0];
  else
    return 0;
}

double LoadAverageMonitor::max()
{
  return max_value;
}

Glib::ustring LoadAverageMonitor::format_value(double val)
{
  return String::ucompose("%1", precision(1), val);
}

Glib::ustring LoadAverageMonitor::get_name()
{
  return _("Load average");
}

Glib::ustring LoadAverageMonitor::get_short_name()
{
  // note to translators: short for "load average" - it has nothing to do with
  // loading data
  return _("Load");
}

void LoadAverageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("load_average"));
}


//
// class MemoryUsageMonitor
//

MemoryUsageMonitor::MemoryUsageMonitor()
  : max_value(0)
{
}

double MemoryUsageMonitor::value()
{
  glibtop_mem mem;
	
  glibtop_get_mem (&mem);

  max_value = mem.total;

  if (mem.total > 0)
    return mem.used - (mem.buffer + mem.cached);
  else
    return 0;
}

double MemoryUsageMonitor::max()
{
  return max_value;
}
	
Glib::ustring MemoryUsageMonitor::format_value(double val)
{
  val /= 1000000;
  
  return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
}

Glib::ustring MemoryUsageMonitor::get_name()
{
  return _("Memory");
}

Glib::ustring MemoryUsageMonitor::get_short_name()
{
  // short for memory
  return _("Mem.");
}

void MemoryUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("memory_usage"));
}


//
// class DiskUsageMonitor
//

DiskUsageMonitor::DiskUsageMonitor(const std::string &dir)
  : max_value(0), mount_dir(dir)
{
}

double DiskUsageMonitor::value()
{
  glibtop_fsusage fsusage;

  glibtop_get_fsusage(&fsusage, mount_dir.c_str());

  max_value = fsusage.blocks;

  if (fsusage.blocks > 0)
    return fsusage.bavail;
  else
    return 0;
}

double DiskUsageMonitor::max()
{
  return max_value;
}

Glib::ustring DiskUsageMonitor::format_value(double val)
{
  // 1 block = disk_block_size bytes
  val *= disk_block_size;

  if (val >= 1000 * 1000 * 1000) {
    val /= 1000 * 1000 * 1000;
    return String::ucompose(_("%1 Gb"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000 * 1000) {
    val /= 1000 * 1000;
    return String::ucompose(_("%1 Mb"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000) {
    val /= 1000;
    return String::ucompose(_("%1 kb"), decimal_digits(val, 3), val);
  }
  else
    return String::ucompose(_("%1 b"), decimal_digits(val, 3), val);
}

Glib::ustring DiskUsageMonitor::get_name()
{
  return String::ucompose(_("Disk (%1)"), mount_dir);
}


Glib::ustring DiskUsageMonitor::get_short_name()
{
  return String::ucompose("%1", mount_dir);
}

void DiskUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			    const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("disk_usage"));
  client->set(dir + "/mount_dir", mount_dir);
}


//
// class NetworkLoadMonitor
//

NetworkLoadMonitor::NetworkLoadMonitor(const Glib::ustring &inter, int inter_no,
				       Direction dir)
  : max_value(1), byte_count(0), time_stamp_secs(0), time_stamp_usecs(0),
    interface(inter), interface_no(inter_no), direction(dir)
{
}

double NetworkLoadMonitor::value()
{
  glibtop_netload netload;

  glibtop_get_netload(&netload,
		      String::ucompose("%1%2", interface, interface_no).c_str());
  guint64 val, bytes;
    
  if (direction == all_data)
    bytes = netload.bytes_total;
  else if (direction == incoming_data)
    bytes = netload.bytes_in;
  else
    bytes = netload.bytes_in;
  
  if (byte_count == 0)
    val = 0;   // avoid calculating an enormous throughput initially
  else
    val = bytes - byte_count;

  byte_count = bytes;

  if (val > max_value)
    max_value = val;

  // calculate difference in msecs
  struct timeval tv;
  if (gettimeofday(&tv, 0) == 0) {
    time_difference =
      (tv.tv_sec - time_stamp_secs) * 1000 +
      (tv.tv_usec - time_stamp_usecs) / 1000;
    time_stamp_secs = tv.tv_sec;
    time_stamp_usecs = tv.tv_usec;
  }

  return val;
}

double NetworkLoadMonitor::max()
{
  return max_value;
}

Glib::ustring NetworkLoadMonitor::format_value(double val)
{
  // 1000 ms = 1 s
  val = val / time_difference * 1000;

  if (val >= 1000 * 1000 * 1000) {
    val /= 1000 * 1000 * 1000;
    return String::ucompose(_("%1 Gb/s"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000 * 1000) {
    val /= 1000 * 1000;
    return String::ucompose(_("%1 Mb/s"), decimal_digits(val, 3), val);
  }
  else if (val >= 1000) {
    val /= 1000;
    return String::ucompose(_("%1 kb/s"), decimal_digits(val, 3), val);
  }
  else 
    return String::ucompose(_("%1 b/s"), decimal_digits(val, 3), val);
}

Glib::ustring NetworkLoadMonitor::get_name()
{
  Glib::ustring str;
  
  if (interface == "eth" && interface_no == 0)
    str = _("Ethernet (first)");
  else if (interface == "eth" && interface_no == 1)
    str = _("Ethernet (second)");
  else if (interface == "eth" && interface_no == 2)
    str = _("Ethernet (third)");
  else if (interface == "ppp" && interface_no == 0)
    str = _("Modem");
  else if (interface == "slip" && interface_no == 0)
    str = _("Serial link");
  else
    // unknown, someone must have been fiddling with GConf
    str = String::ucompose("%1%2", interface, interface_no);

  if (direction == incoming_data)
    // %1 is the network connection, e.g. "Ethernet (first))", in signifies
    // that this is incoming data
    str = String::ucompose(_("%1, in"), str);
  else if (direction == outgoing_data)
    // %1 is the network connection, e.g. "Ethernet (first))", out signifies
    // that this is outgoing data
    str = String::ucompose(_("%1, out"), str);
  
  return str;
}

Glib::ustring NetworkLoadMonitor::get_short_name()
{
  if (interface == "eth")
    // short for an ethernet card
    return String::ucompose(_("Eth. %1"), interface_no + 1);
  else if (interface == "ppp" && interface_no == 0)
    // short for modem
    return _("Mod.");
  else if (interface == "slip" && interface_no == 0)
    // short for serial link
    return _("Ser.");
  else
    // unknown, someone must have been fiddling with GConf
    return String::ucompose("%1%2", interface, interface_no);
}

void NetworkLoadMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
			      const Glib::ustring &dir)
{
  client->set(dir + "/type", Glib::ustring("network_load"));
  client->set(dir + "/interface", interface);
  client->set(dir + "/interface_no", interface_no);
  client->set(dir + "/interface_direction", int(direction));
}
